import React, { Component } from 'react';
import Accordion from 'react-bootstrap/Accordion';
import Button from 'react-bootstrap/Button';
import Dropdown from 'react-bootstrap/Dropdown';
import Offcanvas from 'react-bootstrap/Offcanvas';
import { InfoLg } from 'react-bootstrap-icons';

export default class SettingsTab extends Component {
	constructor(props) {
    super(props);
    this.state = {
      show: false
    }
  }

  handleOpen = () => {
    this.setState({show: true});
  }

  handleClose = () => {
    this.setState({show: false});
  }

  render() {
    return (
      <>
        <Button onClick={this.handleOpen} style={{
          'float': 'right',
          'backgroundColor': 'lightgrey',
          'borderColor': 'lightgrey',
          'fontStyle': 'italic',
          'color': 'grey'
        }}>
            info
        </Button>
        <Offcanvas show={this.state.show} onHide={this.handleClose} placement="end">
          <Offcanvas.Header closeButton>
            <Offcanvas.Title>Instructions and Settings</Offcanvas.Title>
          </Offcanvas.Header>
          <Offcanvas.Body>
            <h6>Instructions</h6>
            <Accordion>
              <Accordion.Item eventKey="0">
                <Accordion.Header>Exploring examples & making example sets</Accordion.Header>
                <Accordion.Body>
                  Examples sets are generated by adding examples to a particular set, and those examples can be found in a variety of ways:  
                  <ul>
                    <li>By selecting example(s) from an existing example set, to search for similar examples, which will appear in the 'search and explore' tab.</li>
                    <li>By searching all examples for those containing a particular word/phrase via the query input in the search and explore tab.</li>
                    <li>By dragging over parts of the 2d projection plot, to populate the search and explore tab.</li>
                    <li>By inputting a custom example in the free text input within an example set (e.g., to use as the initial seed for searching for similar examples).</li>
                  </ul>
                  These methods can be combined – e.g., you might first populate the search and explore tab by selecting a particular area in the plot.  Then, you might select relevant examples to add to an example set, finding more examples via similarity search.  Finally, you could search all examples for a particularly relevant word to see if there are any other examples in the data you may have missed. 
                </Accordion.Body>
              </Accordion.Item>
              <Accordion.Item eventKey="1">
                <Accordion.Header>Tests</Accordion.Header>
                <Accordion.Body>
                  To define a test, click the "+ add new test" button. 
                  You can select from a number of options to define the test.  At the highest level, there’s the test type: 
                  <ul>
                      <li>Output tests check whether the mode has the desired output (e.g., moderated, unmoderated) for a particular example set.</li>
                      <li>Invariance tests specify that predictions should not significantly change between two distributions.</li>
                      <li>Directionality tests specify that predictions should change, and in a specific direction.</li>
                  </ul>
                  Invariance and directionality tests can be computed at different granularities: 
                  <ul>
                      <li>Distributional: a distributional test asks if there is an aggregate change across predictions for two different example sets. </li>
                      <li>Paired: a paired test asks if there is a pairwise change in predictions after applying a transformation (e.g., replacing a particular word) to each example in one example set.</li>
                  </ul>
                  To define a test, select a test type and a granularity, and either example set(s) or transformation function(s) (depending on what the specified test requires). 
                  To run tests, click the “Run all tests” button or the “Re-run test” button within an individual test.   
                </Accordion.Body>
              </Accordion.Item>
            </Accordion>
            <br></br>
            <h6>Current Settings</h6>
            <span><b>Task: </b></span>{this.props.selectedTask}<br></br>
            <span><b>Data: </b></span>{this.props.selectedData}<br></br>
            <span><b>Model: </b></span>{this.props.selectedModel}<br></br>

            <span style={{'display': 'inline-block', 'marginRight': '10px', 'marginTop': '20px', 'font-style': 'italic'}}>Change model: </span> 
            <Dropdown style={{'display': 'inline-block'}}>
              <Dropdown.Toggle id="model-dropdown">{this.props.selectedModel}</Dropdown.Toggle>
              <Dropdown.Menu>
                {this.props.models.map((model) => <Dropdown.Item onClick={() => this.props.changeModel(model)} active={model === this.props.selectedModel}>{model}</Dropdown.Item>)}
              </Dropdown.Menu>
            </Dropdown>

          </Offcanvas.Body>
        </Offcanvas>
      </>
    );
  }
}
